import unittest
import unittest.mock

class HParseResultCreation(unittest.TestCase):
	#def test_valid_address(self):
	#	raise ValueError

	def test_nullpointer(self):
		with self.assertRaises(ValueError):
			res = HParseResult(0)

	def test_make_HParsedToken(self):
		gdbv_patcher = unittest.mock.patch('gdb.Value', autospec=True)
		hpt_patcher = unittest.mock.patch('__main__.HParsedToken', autospec=True)
		hpr_patcher = unittest.mock.patch.object(HParseResult, 'read_AST_not_null', return_value=True)
		rm_patcher = unittest.mock.patch.object(HParseResult, 'read_member', return_value=0xdeadfff0)
		gdbv_mock_object = gdbv_patcher.start()
		hpt_mock_object = hpt_patcher.start()
		hpr_patcher.start()
		rm_mock_object = rm_patcher.start()
		hpr = HParseResult(0xdeadbeef)
		tok = hpr.make_HParsedToken()
		rm_patcher.stop()
		hpr_patcher.stop()
		hpt_patcher.stop()
		gdbv_patcher.stop()
		# Check that read_member('ast') is called
		self.assertEqual(rm_mock_object.call_args, unittest.mock.call('ast'))
		# Check that HParsedToken() is called with the value returned by read_member()
		self.assertEqual(hpt_mock_object.call_args, unittest.mock.call(0xdeadfff0))

	def test_make_HParsedToken_if_no_ast(self):
		gdbv_patcher = unittest.mock.patch('gdb.Value', autospec=True)
		hpt_patcher = unittest.mock.patch('__main__.HParsedToken', autospec=True)
		hpr_patcher = unittest.mock.patch.object(HParseResult, 'read_AST_not_null', return_value=False)
		gdbv_patcher.start()
		hpt_patcher.start()
		hpr_patcher.start()
		hpr = HParseResult(0xdeadbeef)
		tok = hpr.make_HParsedToken()
		hpr_patcher.stop()
		hpt_patcher.stop()
		gdbv_patcher.stop()
		self.assertIsNone(tok)

	def test_read_member(self):
		hpr_pointer_type_patcher = unittest.mock.patch.object(HParseResult, 'HParseResult_t_p', spec=gdb.Type)
		gdbv_patcher = unittest.mock.patch('gdb.Value', autospec=True)
		gdb_lookup_type_patcher = unittest.mock.patch.object(gdb, 'lookup_type')
		init_patcher = unittest.mock.patch.object(HParseResult, '__init__', return_value=None)

		hpr_pointer_type_patcher.start()
		gdbv_mock_object = gdbv_patcher.start()
		gdb_lookup_type_mock_object = gdb_lookup_type_patcher.start()
		init_patcher.start()

		result = HParseResult(0xdeadbeef)
		result.address = 0xdeadbeef
		member = result.read_member('ast')

		init_patcher.stop()
		gdb_lookup_type_patcher.stop()
		gdbv_patcher.stop()
		hpr_pointer_type_patcher.stop()

		# Check that struct being indexed is at self.address
		self.assertEqual(gdbv_mock_object.mock_calls[0], unittest.mock.call(0xdeadbeef))
		# Check that the value returned is parse_result_gdb_value['foo']
		self.assertEqual(gdbv_mock_object.mock_calls[-1], unittest.mock.call().cast().__getitem__('ast'))
		# Check that gdb.lookup_type is not called
		self.assertEqual(gdb_lookup_type_mock_object.mock_calls, [])

	def test_read_member_with_type_lookup(self):
		gdb_lookup_type_patcher = unittest.mock.patch.object(gdb, 'lookup_type')
		gdbv_patcher = unittest.mock.patch('gdb.Value', autospec=True)
		init_patcher = unittest.mock.patch.object(HParseResult, '__init__', return_value=None)
		# This is cached in the class the first time read_member() or read_AST_not_null() is called
		# To test the case where it's not yet cached, we explicitly set it to None
		HParseResult.HParseResult_t_p = None

		gdb_lookup_type_mock_object = gdb_lookup_type_patcher.start()
		gdbv_mock_object = gdbv_patcher.start()
		init_patcher.start()

		result = HParseResult(0xdeadbeef)
		result.address = 0xdeadbeef
		member = result.read_member('ast')

		init_patcher.stop()
		gdbv_patcher.stop()
		gdb_lookup_type_patcher.stop()

		# Check that HParseResult* has been looked up
		self.assertEqual(gdb_lookup_type_mock_object.mock_calls, [unittest.mock.call('HParseResult'), unittest.mock.call().pointer()])
		# Check that the struct being indexed is at self.address
		self.assertEqual(gdbv_mock_object.mock_calls[0], unittest.mock.call(0xdeadbeef))
		# Check that the value returned is parse_result_gdb_value['foo']
		self.assertEqual(gdbv_mock_object.mock_calls[-1], unittest.mock.call().cast().__getitem__('ast'))

	# TODO: should read_member employ a whitelist for member_name, or should this trigger an exeption?
	def test_read_member_invalid_param(self):
		raise Exception("Not implemented")
